import pandas as pd
import numpy as np
from sklearn.preprocessing import KBinsDiscretizer
from typing import Dict, List, Tuple
import matplotlib.pyplot as plt
import seaborn as sns

class MultiFeatureRuleMiner:
    """
    多特征交叉规则生成类，用于生成双特征交叉分析结果
    """
    
    def __init__(self, df: pd.DataFrame, target_col: str = 'ISBAD'):
        """
        初始化多特征规则挖掘器
        
        参数:
            df: 输入的数据集
            target_col: 目标字段名，默认为'ISBAD'
        """
        self.df = df.copy()
        self.target_col = target_col
        
        # 筛选出数值型特征
        self.numeric_features = [col for col in self.df.columns 
                                if col != target_col and pd.api.types.is_numeric_dtype(self.df[col])]
        
        # 筛选出类别型特征
        self.categorical_features = [col for col in self.df.columns 
                                    if col != target_col and not pd.api.types.is_numeric_dtype(self.df[col])]
        
    def _bin_numeric_feature(self, feature: str, max_bins: int = 20) -> pd.Series:
        """
        对数值型特征进行分箱处理
        
        参数:
            feature: 特征名
            max_bins: 最大分箱数量，默认为20
            
        返回:
            分箱后的特征值
        """
        data = self.df[feature].dropna().values.reshape(-1, 1)
        discretizer = KBinsDiscretizer(n_bins=max_bins, encode='ordinal', strategy='quantile', quantile_method='averaged_inverted_cdf')
        binned = discretizer.fit_transform(data)
        
        # 转换为Series，保留原始索引
        binned_series = pd.Series(binned.flatten(), index=self.df[feature].dropna().index)
        
        # 填充缺失值
        result = pd.Series(index=self.df.index, dtype=float)
        result.loc[binned_series.index] = binned_series
        
        return result
    
    def _prepare_feature(self, feature: str, max_unique_values: int = 20) -> pd.Series:
        """
        准备特征，对取值较多的特征进行分箱处理
        
        参数:
            feature: 特征名
            max_unique_values: 最大允许的唯一值数量，超过则进行分箱
            
        返回:
            处理后的特征值
        """
        unique_count = self.df[feature].nunique()
        
        if unique_count > max_unique_values:
            # 对数值型特征进行分箱
            if feature in self.numeric_features:
                return self._bin_numeric_feature(feature, max_unique_values)
            # 对类别型特征，保留前max_unique_values个最常见的值，其余归为"其他"
            else:
                top_values = self.df[feature].value_counts().head(max_unique_values).index
                return self.df[feature].apply(lambda x: x if x in top_values else '其他')
        else:
            return self.df[feature]
    
    def generate_cross_matrix(self, feature1: str, feature2: str, 
                             max_unique_values: int = 20) -> pd.DataFrame:
        """
        生成双特征交叉矩阵，包含badrate、样本占比等关键指标
        
        参数:
            feature1: 第一个特征名
            feature2: 第二个特征名
            max_unique_values: 最大允许的唯一值数量，超过则进行分箱
            
        返回:
            交叉矩阵，包含各种统计指标
        """
        # 准备两个特征
        prepared_feature1 = self._prepare_feature(feature1, max_unique_values)
        prepared_feature2 = self._prepare_feature(feature2, max_unique_values)
        
        # 创建交叉表数据
        cross_data = pd.DataFrame({
            'feature1': prepared_feature1,
            'feature2': prepared_feature2,
            'target': self.df[self.target_col]
        })
        
        # 计算基本统计量
        # 计算每个交叉组合的样本数
        count_matrix = pd.crosstab(cross_data['feature1'], cross_data['feature2'], 
                                  rownames=[feature1], colnames=[feature2])
        
        # 计算每个交叉组合的坏样本数
        bad_count_matrix = pd.crosstab(cross_data['feature1'], cross_data['feature2'], 
                                      values=cross_data['target'], aggfunc='sum',
                                      rownames=[feature1], colnames=[feature2])
        
        # 计算每个交叉组合的坏样本率
        badrate_matrix = bad_count_matrix / count_matrix
        
        # 计算每个交叉组合的样本占比
        total_samples = count_matrix.sum().sum()
        sample_ratio_matrix = count_matrix / total_samples
        
        # 计算lift值
        # 总样本坏样本率
        total_badrate = self.df[self.target_col].mean()
        lift_matrix = badrate_matrix / total_badrate
        
        # 整合所有矩阵到一个MultiIndex DataFrame中
        # 创建一个空的MultiIndex DataFrame
        features = [feature1, feature2]
        metrics = ['count', 'bad_count', 'badrate', 'sample_ratio', 'lift']
        
        # 生成行索引（feature1的唯一值）
        rows = count_matrix.index
        # 生成列索引（feature2的唯一值 + 指标）
        cols = pd.MultiIndex.from_product([count_matrix.columns, metrics], 
                                         names=[feature2, 'metric'])
        
        # 填充数据
        cross_matrix = pd.DataFrame(index=rows, columns=cols)
        
        for f2_val in count_matrix.columns:
            cross_matrix[(f2_val, 'count')] = count_matrix[f2_val]
            cross_matrix[(f2_val, 'bad_count')] = bad_count_matrix[f2_val]
            cross_matrix[(f2_val, 'badrate')] = badrate_matrix[f2_val]
            cross_matrix[(f2_val, 'sample_ratio')] = sample_ratio_matrix[f2_val]
            cross_matrix[(f2_val, 'lift')] = lift_matrix[f2_val]
        
        return cross_matrix
    
    def get_cross_rules(self, feature1: str, feature2: str, 
                       top_n: int = 10, metric: str = 'lift',
                       max_unique_values: int = 20) -> pd.DataFrame:
        """
        从交叉矩阵中提取top规则
        
        参数:
            feature1: 第一个特征名
            feature2: 第二个特征名
            top_n: 返回的规则数量，默认为10
            metric: 排序指标，默认为'lift'
            max_unique_values: 最大允许的唯一值数量，超过则进行分箱
            
        返回:
            包含top规则的DataFrame
        """
        # 生成交叉矩阵
        cross_matrix = self.generate_cross_matrix(feature1, feature2, max_unique_values)
        
        # 转换为长格式，方便排序
        long_df = cross_matrix.stack(level=0, future_stack=True).reset_index()
        long_df.rename(columns={feature1: 'feature1_value', feature2: 'feature2_value'}, inplace=True)
        
        # 按指定指标排序，返回top_n
        top_rules = long_df.sort_values(by=metric, ascending=False).head(top_n)
        
        # 添加规则描述
        top_rules['rule_description'] = top_rules.apply(
            lambda x: f"{feature1} = {x['feature1_value']} AND {feature2} = {x['feature2_value']}",
            axis=1
        )
        
        return top_rules
    
    def plot_cross_heatmap(self, feature1: str, feature2: str, 
                          metric: str = 'lift', max_unique_values: int = 20, 
                          figsize: Tuple[int, int] = (12, 10)):
        """
        绘制双特征交叉热力图
        
        参数:
            feature1: 第一个特征名
            feature2: 第二个特征名
            metric: 要可视化的指标，默认为'lift'
            max_unique_values: 最大允许的唯一值数量，超过则进行分箱
            figsize: 图表大小
            
        返回:
            matplotlib.pyplot对象
        """
        # 生成交叉矩阵
        cross_matrix = self.generate_cross_matrix(feature1, feature2, max_unique_values)
        
        # 提取指定指标的矩阵
        metric_matrix = cross_matrix.xs(metric, level='metric', axis=1)
        
        # 创建热力图
        plt.figure(figsize=figsize)
        sns.heatmap(metric_matrix, annot=True, fmt='.3f', cmap='coolwarm', 
                   cbar_kws={'label': metric})
        
        plt.title(f'{feature1} vs {feature2} - {metric} Heatmap')
        plt.tight_layout()
        
        return plt
    
    def generate_all_cross_rules(self, top_n: int = 10, metric: str = 'lift',
                                max_unique_values: int = 20, max_feature_pairs: int = 10) -> pd.DataFrame:
        """
        生成所有特征对的交叉规则，并返回top规则
        
        参数:
            top_n: 返回的规则数量，默认为10
            metric: 排序指标，默认为'lift'
            max_unique_values: 最大允许的唯一值数量，超过则进行分箱
            max_feature_pairs: 最大生成的特征对数量，默认为10
            
        返回:
            包含top规则的DataFrame
        """
        # 生成特征对组合
        from itertools import combinations
        features = self.numeric_features + self.categorical_features
        feature_pairs = list(combinations(features, 2))[:max_feature_pairs]  # 限制特征对数量
        
        all_rules = []
        
        for feature1, feature2 in feature_pairs:
            try:
                rules = self.get_cross_rules(feature1, feature2, top_n=top_n, 
                                            metric=metric, max_unique_values=max_unique_values)
                if not rules.empty:
                    all_rules.append(rules)
            except Exception as e:
                print(f"Error generating rules for {feature1} and {feature2}: {e}")
                continue
        
        if not all_rules:
            return pd.DataFrame()
        
        # 合并所有规则，取top_n
        combined_rules = pd.concat(all_rules, ignore_index=True)
        return combined_rules.sort_values(by=metric, ascending=False).head(top_n)



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
from typing import Dict, List, Tuple, Optional, Union
import networkx as nx
from sklearn.tree import export_graphviz
import graphviz

class RuleVisualizer:
    """
    规则可视化类，用于生成各种规则可视化图表
    """
    
    def __init__(self, dpi: int = 300):
        """
        初始化规则可视化器
        
        参数:
            dpi: 图像分辨率，默认为300
        """
        self.dpi = dpi
        
        # 设置中文字体支持
        plt.rcParams['font.sans-serif'] = ['SimHei', 'Arial Unicode MS', 'DejaVu Sans']
        plt.rcParams['axes.unicode_minus'] = False
        
    def plot_rule_comparison(self, rules_df: pd.DataFrame, metrics: List[str] = ['lift', 'badrate', 'sample_count'],
                           figsize: Tuple[int, int] = (15, 10), save_path: Optional[str] = None) -> plt.Axes:
        """
        绘制规则比较图
        
        参数:
            rules_df: 包含规则信息的DataFrame
            metrics: 要比较的指标列表，默认为['lift', 'badrate', 'sample_count']
            figsize: 图表大小，默认为(15, 10)
            save_path: 保存路径，如'./rule_comparison.png'，默认为None
            
        返回:
            图表对象
        """
        # 验证规则DataFrame包含必要列
        if 'rule_description' not in rules_df.columns:
            raise ValueError("规则DataFrame必须包含'rule_description'列")
        
        for metric in metrics:
            if metric not in rules_df.columns:
                raise ValueError(f"规则DataFrame必须包含{metric}列")
        
        # 创建子图
        n_metrics = len(metrics)
        fig, axes = plt.subplots(n_metrics, 1, figsize=figsize, sharex=True)
        
        if n_metrics == 1:
            axes = [axes]
        
        # 绘制每个指标的条形图
        for i, metric in enumerate(metrics):
            # 修复FutureWarning：将x变量分配给hue
            sns.barplot(x='rule_description', y=metric, hue='rule_description', data=rules_df, ax=axes[i], palette='viridis', dodge=False)
            axes[i].set_ylabel(metric.capitalize())
            axes[i].set_title(f'Rule Comparison - {metric.capitalize()}')
            # 设置x轴标签旋转和对齐
            axes[i].tick_params(axis='x', rotation=45)
            plt.setp(axes[i].get_xticklabels(), ha='right')
            
            # 添加数值标签
            for p in axes[i].patches:
                height = p.get_height()
                axes[i].annotate(f'{height:.4f}',
                               xy=(p.get_x() + p.get_width() / 2., height),
                               xytext=(0, 3),  # 3 points vertical offset
                               textcoords='offset points',
                               ha='center', va='bottom',
                               fontsize=8)
        
        plt.tight_layout()
        
        # 保存图表
        if save_path:
            plt.savefig(save_path, dpi=self.dpi, bbox_inches='tight')
        
        return fig
    
    def plot_rule_distribution(self, rules_df: pd.DataFrame, metric: str = 'lift',
                             figsize: Tuple[int, int] = (10, 6), save_path: Optional[str] = None) -> plt.Axes:
        """
        绘制规则分布直方图
        
        参数:
            rules_df: 包含规则信息的DataFrame
            metric: 要绘制分布的指标，默认为'lift'
            figsize: 图表大小，默认为(10, 6)
            save_path: 保存路径，如'./rule_distribution.png'，默认为None
            
        返回:
            图表对象
        """
        # 验证规则DataFrame包含必要列
        if metric not in rules_df.columns:
            raise ValueError(f"规则DataFrame必须包含{metric}列")
        
        # 创建图表
        fig, ax = plt.subplots(figsize=figsize)
        
        # 绘制直方图
        sns.histplot(rules_df[metric], kde=True, ax=ax, bins=20, color='skyblue', edgecolor='black')
        
        # 添加统计信息
        mean_val = rules_df[metric].mean()
        median_val = rules_df[metric].median()
        std_val = rules_df[metric].std()
        
        ax.axvline(mean_val, color='red', linestyle='--', label=f'Mean: {mean_val:.4f}')
        ax.axvline(median_val, color='green', linestyle='--', label=f'Median: {median_val:.4f}')
        
        ax.set_title(f'Distribution of Rule {metric.capitalize()}')
        ax.set_xlabel(metric.capitalize())
        ax.set_ylabel('Frequency')
        ax.legend()
        ax.grid(True, alpha=0.3)
        
        plt.tight_layout()
        
        # 保存图表
        if save_path:
            plt.savefig(save_path, dpi=self.dpi, bbox_inches='tight')
        
        return fig
    
    def plot_lift_precision_scatter(self, rules_df: pd.DataFrame, size_col: str = 'sample_count',
                                  figsize: Tuple[int, int] = (10, 8), save_path: Optional[str] = None) -> plt.Axes:
        """
        绘制Lift-Precision散点图
        
        参数:
            rules_df: 包含规则信息的DataFrame
            size_col: 用于点大小的列，默认为'sample_count'
            figsize: 图表大小，默认为(10, 8)
            save_path: 保存路径，如'./lift_precision_scatter.png'，默认为None
            
        返回:
            图表对象
        """
        # 验证规则DataFrame包含必要列
        for col in ['lift', 'precision', size_col]:
            if col not in rules_df.columns:
                raise ValueError(f"规则DataFrame必须包含{col}列")
        
        # 创建图表
        fig, ax = plt.subplots(figsize=figsize)
        
        # 绘制散点图
        scatter = ax.scatter(rules_df['precision'], rules_df['lift'], 
                           s=rules_df[size_col] / rules_df[size_col].max() * 1000, 
                           alpha=0.7, c=rules_df['badrate'], cmap='viridis')
        
        # 添加颜色条
        cbar = fig.colorbar(scatter)
        cbar.set_label('Badrate')
        
        ax.set_title('Lift vs Precision Scatter Plot')
        ax.set_xlabel('Precision')
        ax.set_ylabel('Lift')
        ax.grid(True, alpha=0.3)
        
        # 添加平均线
        ax.axhline(rules_df['lift'].mean(), color='red', linestyle='--', alpha=0.5)
        ax.axvline(rules_df['precision'].mean(), color='red', linestyle='--', alpha=0.5)
        
        plt.tight_layout()
        
        # 保存图表
        if save_path:
            plt.savefig(save_path, dpi=self.dpi, bbox_inches='tight')
        
        return fig
    
    def plot_decision_tree(self, model: any, feature_cols: List[str], class_names: List[str] = ['Good', 'Bad'],
                         max_depth: int = 5, figsize: Tuple[int, int] = (20, 15), save_path: Optional[str] = None) -> plt.Axes:
        """
        绘制决策树
        
        参数:
            model: 训练好的决策树模型
            feature_cols: 特征列名列表
            class_names: 类别名称列表，默认为['Good', 'Bad']
            max_depth: 绘制的最大深度，默认为5
            figsize: 图表大小，默认为(20, 15)
            save_path: 保存路径，如'./decision_tree.png'，默认为None
            
        返回:
            图表对象
        """
        # 使用graphviz绘制决策树
        dot_data = export_graphviz(
            model,
            out_file=None,
            feature_names=feature_cols,
            class_names=class_names,
            filled=True,
            rounded=True,
            special_characters=True,
            max_depth=max_depth
        )
        
        # 生成图表
        graph = graphviz.Source(dot_data)
        
        # 保存图表
        if save_path:
            graph.render(filename=save_path.replace('.png', ''), format='png', cleanup=True)
        
        return graph
    
    def plot_rule_network(self, rules: List[Dict[str, any]], figsize: Tuple[int, int] = (15, 15),
                        save_path: Optional[str] = None) -> plt.Axes:
        """
        绘制规则网络图
        
        参数:
            rules: 包含规则信息的字典列表
            figsize: 图表大小，默认为(15, 15)
            save_path: 保存路径，如'./rule_network.png'，默认为None
            
        返回:
            图表对象
        """
        # 创建有向图
        G = nx.DiGraph()
        
        # 添加节点
        for i, rule in enumerate(rules):
            # 计算节点大小，基于样本数和重要性
            node_size = rule.get('samples', 1) * rule.get('importance', 1)
            node_color = rule.get('badrate', 0)  # 使用坏账率作为节点颜色
            
            G.add_node(f'Rule {i+1}', 
                      size=node_size,
                      color=node_color,
                      lift=rule.get('lift', 1),
                      badrate=rule.get('badrate', 0),
                      precision=rule.get('precision', 0))
        
        # 添加边（这里简单模拟，实际应根据规则间的关联关系）
        for i in range(len(rules)):
            for j in range(i+1, len(rules)):
                # 简单模拟边权重，基于规则lift值的差异
                weight = abs(rules[i].get('lift', 1) - rules[j].get('lift', 1))
                if weight > 0.5:  # 只添加权重较大的边
                    G.add_edge(f'Rule {i+1}', f'Rule {j+1}', weight=weight)
        
        # 绘制网络图
        fig, ax = plt.subplots(figsize=figsize)
        
        # 计算节点位置
        pos = nx.spring_layout(G, k=0.3, iterations=50)
        
        # 获取节点大小和颜色
        node_sizes = [G.nodes[node]['size'] * 10 for node in G.nodes()]
        node_colors = [G.nodes[node]['color'] for node in G.nodes()]
        
        # 绘制节点
        nx.draw_networkx_nodes(G, pos, ax=ax, node_size=node_sizes, node_color=node_colors, 
                             cmap='viridis', alpha=0.8)
        
        # 绘制边
        edges = nx.draw_networkx_edges(G, pos, ax=ax, edge_color='gray', alpha=0.5, arrowsize=10)
        
        # 绘制节点标签
        nx.draw_networkx_labels(G, pos, ax=ax, font_size=8, font_weight='bold')
        
        # 添加颜色条
        sm = plt.cm.ScalarMappable(cmap='viridis', norm=plt.Normalize(vmin=min(node_colors), vmax=max(node_colors)))
        sm.set_array([])
        cbar = fig.colorbar(sm, ax=ax)
        cbar.set_label('Badrate')
        
        ax.set_title('Rule Network Visualization')
        ax.axis('off')
        
        plt.tight_layout()
        
        # 保存图表
        if save_path:
            plt.savefig(save_path, dpi=self.dpi, bbox_inches='tight')
        
        return fig
    
    def plot_heatmap(self, data: pd.DataFrame, figsize: Tuple[int, int] = (12, 10), save_path: Optional[str] = None) -> plt.Axes:
        """
        绘制热力图
        
        参数:
            data: 要绘制热力图的数据，通常是相关性矩阵或交叉表
            figsize: 图表大小，默认为(12, 10)
            save_path: 保存路径，如'./heatmap.png'，默认为None
            
        返回:
            图表对象
        """
        # 创建图表
        fig, ax = plt.subplots(figsize=figsize)
        
        # 绘制热力图
        sns.heatmap(data, annot=True, fmt='.4f', cmap='RdYlBu', ax=ax, square=True)
        
        ax.set_title('Correlation Heatmap')
        
        plt.tight_layout()
        
        # 保存图表
        if save_path:
            plt.savefig(save_path, dpi=self.dpi, bbox_inches='tight')
        
        return fig
    
    def export_rules(self, rules_df: pd.DataFrame, export_path: str, export_format: str = 'csv'):
        """
        导出规则信息
        
        参数:
            rules_df: 包含规则信息的DataFrame
            export_path: 导出路径，如'./rules'
            export_format: 导出格式，支持'csv'、'json'、'excel'，默认为'csv'
        """
        if export_format == 'csv':
            rules_df.to_csv(f'{export_path}.csv', index=False, encoding='utf-8-sig')
        elif export_format == 'json':
            rules_df.to_json(f'{export_path}.json', orient='records', force_ascii=False, indent=2)
        elif export_format == 'excel':
            rules_df.to_excel(f'{export_path}.xlsx', index=False, engine='openpyxl')
        else:
            raise ValueError(f"不支持的导出格式: {export_format}，支持的格式有: 'csv', 'json', 'excel'")
    
    def generate_rule_report(self, rules_df: pd.DataFrame, report_path: str = './rule_report'):
        """
        生成规则报告，包含多种可视化图表
        
        参数:
            rules_df: 包含规则信息的DataFrame
            report_path: 报告保存路径，默认为'./rule_report'
        """
        # 生成规则比较图
        self.plot_rule_comparison(rules_df, save_path=f'{report_path}_comparison.png')
        
        # 生成规则分布图
        self.plot_rule_distribution(rules_df, save_path=f'{report_path}_distribution.png')
        
        # 如果包含lift和precision列，生成散点图
        if 'lift' in rules_df.columns and 'precision' in rules_df.columns:
            self.plot_lift_precision_scatter(rules_df, save_path=f'{report_path}_lift_precision.png')
        
        # 导出规则数据
        self.export_rules(rules_df, report_path, export_format='csv')
        self.export_rules(rules_df, report_path, export_format='json')
        self.export_rules(rules_df, report_path, export_format='excel')

# 简化的API函数
def plot_rule_comparison(rules_df: pd.DataFrame, metrics: List[str] = ['lift', 'badrate', 'sample_count'],
                        figsize: Tuple[int, int] = (15, 10), save_path: Optional[str] = None) -> plt.Axes:
    """
    绘制规则比较图的简化API
    """
    visualizer = RuleVisualizer()
    return visualizer.plot_rule_comparison(rules_df, metrics, figsize, save_path)

def plot_rule_distribution(rules_df: pd.DataFrame, metric: str = 'lift',
                          figsize: Tuple[int, int] = (10, 6), save_path: Optional[str] = None) -> plt.Axes:
    """
    绘制规则分布直方图的简化API
    """
    visualizer = RuleVisualizer()
    return visualizer.plot_rule_distribution(rules_df, metric, figsize, save_path)

def plot_lift_precision_scatter(rules_df: pd.DataFrame, size_col: str = 'sample_count',
                               figsize: Tuple[int, int] = (10, 8), save_path: Optional[str] = None) -> plt.Axes:
    """
    绘制Lift-Precision散点图的简化API
    """
    visualizer = RuleVisualizer()
    return visualizer.plot_lift_precision_scatter(rules_df, size_col, figsize, save_path)

def plot_decision_tree(model: any, feature_cols: List[str], class_names: List[str] = ['Good', 'Bad'],
                      max_depth: int = 5, figsize: Tuple[int, int] = (20, 15), save_path: Optional[str] = None) -> plt.Axes:
    """
    绘制决策树的简化API
    """
    visualizer = RuleVisualizer()
    return visualizer.plot_decision_tree(model, feature_cols, class_names, max_depth, figsize, save_path)

def plot_rule_network(rules: List[Dict[str, any]], figsize: Tuple[int, int] = (15, 15),
                     save_path: Optional[str] = None) -> plt.Axes:
    """
    绘制规则网络图的简化API
    """
    visualizer = RuleVisualizer()
    return visualizer.plot_rule_network(rules, figsize, save_path)

def plot_heatmap(data: pd.DataFrame, figsize: Tuple[int, int] = (12, 10), save_path: Optional[str] = None) -> plt.Axes:
    """
    绘制热力图的简化API
    """
    visualizer = RuleVisualizer()
    return visualizer.plot_heatmap(data, figsize, save_path)

def generate_rule_report(rules_df: pd.DataFrame, report_path: str = './rule_report'):
    """
    生成规则报告的简化API
    """
    visualizer = RuleVisualizer()
    visualizer.generate_rule_report(rules_df, report_path)



import pandas as pd
import numpy as np
from typing import List, Dict, Any, Tuple
from sklearn.preprocessing import KBinsDiscretizer
import matplotlib.pyplot as plt
import seaborn as sns

class SingleFeatureRuleMiner:
    """
    单特征规则挖掘类，用于对数据各特征的不同阈值进行效度分布分析
    
    Attributes:
        df: 输入的数据集
        exclude_cols: 排除的字段名列表
        target_col: 目标字段名，默认为'ISBAD'
        features: 待分析的特征列表
    """
    
    def __init__(self, df: pd.DataFrame, exclude_cols: List[str] = None, target_col: str = 'ISBAD'):
        """
        初始化单特征规则挖掘器
        
        参数:
            df: 输入的数据集
            exclude_cols: 排除的字段名列表
            target_col: 目标字段名，默认为'ISBAD'
        """
        self.df = df.copy()
        self.target_col = target_col
        
        # 排除指定字段和非数值字段
        self.exclude_cols = exclude_cols if exclude_cols else []
        self.exclude_cols.append(self.target_col)
        
        # 筛选出数值型特征
        self.features = [col for col in self.df.columns 
                         if col not in self.exclude_cols 
                         and pd.api.types.is_numeric_dtype(self.df[col])]
    
    def _calculate_metrics(self, feature: str, threshold: float, operator: str = '>=') -> Dict[str, float]:
        """
        计算单个阈值下的统计指标
        
        参数:
            feature: 特征名
            threshold: 阈值
            operator: 操作符，支持'>='或'<='
            
        返回:
            包含各类统计指标的字典
        """
        # 根据操作符筛选数据
        if operator == '>=':
            selected = self.df[self.df[feature] >= threshold]
        else:  # '<='
            selected = self.df[self.df[feature] <= threshold]
        
        total = len(self.df)
        selected_count = len(selected)
        
        # 总样本中的坏样本数
        total_bad = self.df[self.target_col].sum()
        # 选中样本中的坏样本数
        selected_bad = selected[self.target_col].sum()
        
        # 计算指标
        hit_rate = selected_count / total if total > 0 else 0  # 命中率
        badrate = selected_bad / selected_count if selected_count > 0 else 0  # 坏账率
        
        # 总坏账率
        total_badrate = total_bad / total if total > 0 else 0
        
        # lift值
        lift = badrate / total_badrate if total_badrate > 0 else 0
        
        # 召回率
        recall = selected_bad / total_bad if total_bad > 0 else 0
        
        # 精确率
        precision = selected_bad / selected_count if selected_count > 0 else 0
        
        # F1分数
        f1 = 2 * (precision * recall) / (precision + recall) if (precision + recall) > 0 else 0
        
        return {
            'feature': feature,
            'threshold': threshold,
            'operator': operator,
            'total_samples': total,
            'selected_samples': selected_count,
            'hit_rate': hit_rate,
            'total_bad': total_bad,
            'selected_bad': selected_bad,
            'badrate': badrate,
            'total_badrate': total_badrate,
            'lift': lift,
            'recall': recall,
            'precision': precision,
            'f1': f1
        }
    
    def analyze_feature(self, feature: str, n_bins: int = 20) -> pd.DataFrame:
        """
        分析单个特征的不同阈值下的效度分布
        
        参数:
            feature: 特征名
            n_bins: 分箱数量，默认为20
            
        返回:
            包含所有阈值指标的DataFrame
        """
        if feature not in self.features:
            raise ValueError(f"Feature {feature} is not in the list of numeric features.")
        
        # 对特征值进行分箱，获取阈值列表
        data = self.df[feature].dropna().values.reshape(-1, 1)
        discretizer = KBinsDiscretizer(n_bins=n_bins, encode='ordinal', strategy='quantile', quantile_method='averaged_inverted_cdf')
        discretizer.fit(data)
        
        # 获取分箱边界作为阈值
        thresholds = sorted(set(discretizer.bin_edges_[0]))
        
        # 对每个阈值计算>=和<=两种情况的指标
        results = []
        for threshold in thresholds:
            results.append(self._calculate_metrics(feature, threshold, '>='))
            results.append(self._calculate_metrics(feature, threshold, '<='))
        
        return pd.DataFrame(results)
    
    def analyze_all_features(self, n_bins: int = 20) -> Dict[str, pd.DataFrame]:
        """
        分析所有特征的不同阈值下的效度分布
        
        参数:
            n_bins: 分箱数量，默认为20
            
        返回:
            字典，键为特征名，值为包含该特征所有阈值指标的DataFrame
        """
        results = {}
        for feature in self.features:
            results[feature] = self.analyze_feature(feature, n_bins)
        return results
    
    def plot_threshold_analysis(self, feature: str, metric: str = 'lift', n_bins: int = 20, 
                               figsize: Tuple[int, int] = (12, 6)):
        """
        绘制特征阈值分析图
        
        参数:
            feature: 特征名
            metric: 要可视化的指标，默认为'lift'
            n_bins: 分箱数量，默认为20
            figsize: 图表大小
        """
        # 分析特征
        df_result = self.analyze_feature(feature, n_bins)
        
        # 创建图表
        plt.figure(figsize=figsize)
        
        # 分别绘制>=和<=两种情况
        for operator in ['>=', '<=']:
            subset = df_result[df_result['operator'] == operator]
            plt.plot(subset['threshold'], subset[metric], label=f'{operator}', marker='o', markersize=5)
        
        plt.title(f'Feature {feature} - {metric} Analysis')
        plt.xlabel(f'{feature} Threshold')
        plt.ylabel(metric.capitalize())
        plt.grid(True, alpha=0.3)
        plt.legend()
        plt.tight_layout()
        
        return plt
    
    def plot_badrate_vs_hitrate(self, feature: str, n_bins: int = 20, 
                               figsize: Tuple[int, int] = (12, 6)):
        """
        绘制坏账率 vs 命中率散点图
        
        参数:
            feature: 特征名
            n_bins: 分箱数量，默认为20
            figsize: 图表大小
        """
        # 分析特征
        df_result = self.analyze_feature(feature, n_bins)
        
        # 创建图表
        plt.figure(figsize=figsize)
        
        # 分别绘制>=和<=两种情况
        for operator in ['>=', '<=']:
            subset = df_result[df_result['operator'] == operator]
            plt.scatter(subset['hit_rate'], subset['badrate'], 
                       label=f'{operator}', alpha=0.7, s=50)
        
        plt.title(f'Feature {feature} - Badrate vs Hitrate')
        plt.xlabel('Hit Rate')
        plt.ylabel('Badrate')
        plt.grid(True, alpha=0.3)
        plt.legend()
        plt.tight_layout()
        
        return plt
    
    # 兼容example.py的别名方法
    def calculate_single_feature_metrics(self, feature: str, num_bins: int = 20) -> pd.DataFrame:
        """
        计算单个特征的不同阈值的效度指标（别名方法，兼容example.py）
        
        参数:
            feature: 特征名
            num_bins: 分箱数量，默认为20
            
        返回:
            包含各阈值统计指标的DataFrame
        """
        return self.analyze_feature(feature, n_bins=num_bins)
    
    def plot_feature_metrics(self, feature: str, metric: str = 'lift', num_bins: int = 20, 
                           figsize: Tuple[int, int] = (12, 8)):
        """
        绘制单个特征的指标分布图（别名方法，兼容example.py）
        
        参数:
            feature: 特征名
            metric: 要可视化的指标，默认为'lift'
            num_bins: 分箱数量，默认为20
            figsize: 图表大小
            
        返回:
            matplotlib.pyplot对象
        """
        return self.plot_threshold_analysis(feature, metric=metric, n_bins=num_bins, figsize=figsize)
    
    def get_top_rules(self, feature: str = None, top_n: int = 10, metric: str = 'lift') -> pd.DataFrame:
        """
        获取单个特征的top规则或所有特征的top规则
        
        参数:
            feature: 特征名，默认为None（获取所有特征的top规则）
            top_n: 返回的规则数量，默认为10
            metric: 排序指标，默认为'lift'
            
        返回:
            包含top规则的DataFrame
        """
        if feature is not None:
            # 获取单个特征的top规则
            feature_results = self.analyze_feature(feature)
            top_rules = feature_results.sort_values(by=metric, ascending=False).head(top_n)
            
            # 添加规则描述
            top_rules['rule_description'] = top_rules.apply(
                lambda x: f"{feature} {x['operator']} {x['threshold']:.4f}",
                axis=1
            )
            
            # 添加sample_ratio列作为hit_rate的别名
            top_rules['sample_ratio'] = top_rules['hit_rate']
            
            return top_rules
        else:
            # 获取所有特征的top规则
            all_results = self.analyze_all_features()
            df_all = pd.concat(all_results.values(), ignore_index=True)
            top_rules = df_all.sort_values(by=metric, ascending=False).head(top_n)
            
            # 添加规则描述
            top_rules['rule_description'] = top_rules.apply(
                lambda x: f"{x['feature']} {x['operator']} {x['threshold']:.4f}",
                axis=1
            )
            
            # 添加sample_ratio列作为hit_rate的别名
            top_rules['sample_ratio'] = top_rules['hit_rate']
            
            return top_rules
